//###########################################################################################
// RF3Sens (RoboForum triangulation range sensor)
// http://roboforum.ru/forum107/topic15929.html (official language for this forum - russian)
// MIT License
// 2015-2016
// Дмитрий (aka Dmitry__ @ RoboForum.ru)
// Андрей Пожогин (aka dccharacter  @ RoboForum.ru)
// Сергей Тараненко (aka setar @ RoboForum.ru)
//###########################################################################################

#include "Config.h"
#if defined(debug_type) && defined(software_serial)
  #include <SendOnlySoftwareSerial.h>
#endif

unsigned char Str[5];

void setup(){
  //set pin I/O direction
  #ifdef use_nCS
    pin_nCS_Mode_OUTPUT;
    pin_nCS_HIGH;
  #endif
  pin_led_Mode_OUTPUT;
  pin_led_HIGH;
#ifdef pin_TRIG_bit
  pin_TRIG_Mode_INPUT;
#endif

#ifdef power_via_mcu
  #ifdef pin_sensor_gnd_bit
    pin_sensor_gnd_OUTPUT;
    pin_sensor_gnd_LOW;
  #endif
  #ifdef pin_sensor_vcc_bit
    pin_sensor_vcc_OUTPUT;
    pin_sensor_vcc_HIGH;
  #endif
  #ifdef pin_laser_gnd_bit
    pin_laser_gnd_OUTPUT;
    pin_laser_gnd_LOW;
  #endif
  #ifdef pin_laser_vcc_bit
    pin_laser_vcc_OUTPUT;
    pin_laser_vcc_HIGH;
  #endif
#endif // power_via_mcu

//initialize SPI
pin_nClock_Mode_OUTPUT;
pin_nClock_HIGH;
pin_SDIO_Mode_OUTPUT;
pin_SDIO_LOW;

#if defined(debug_type)
  #if !defined(software_serial)
    Serial.begin(230400);
  #else
    MyDbgSerial.begin(115200);
    //MyDbgSerial.println(F("\nDebug enabled"));
  #endif
#endif

  delay(1000);
  ADNS_reset();
}

void loop(){
//###########################################################################################
// штатный режим датчика для 3D принтера
#ifndef debug_type
  byte dataMax, dataMin, dataPix_Sum;

  while(1){
    dataMax = ADNS_read(Maximum_Pixel);
    //dataMax = ADNS_read(Pixel_Sum);
    //dataMax = ADNS_read(squal);

    dataMax > ConstMax ? pin_led_LOW : pin_led_HIGH;

/*
    pin_led_HIGH;
    while(1){ //шаг1
      dataMax = ADNS_read(Maximum_Pixel);
      if(dataMax > ConstMax) break;
    }
    while(1){ //шаг2
      dataMax = ADNS_read(Maximum_Pixel);
      dataMin = ADNS_read(Minimum_Pixel);
      if(dataMax >(ConstMax -2) && dataMin < ConstMin) break;
    }
    while(1){ //шаг3
      dataMax = ADNS_read(Maximum_Pixel);
      dataMin = ADNS_read(Minimum_Pixel);
      dataPix_Sum = ADNS_read(Pixel_Sum);
      if(dataMax > (ConstMax -2) && dataMin < (ConstMin +2) && dataPix_Sum > ConstPixMin && dataPix_Sum < ConstPixMax) break;
    }
    pin_led_LOW;

    while(1){ //ожидание подьема головы
      dataMax = ADNS_read(Maximum_Pixel);
      if(dataMax < ConstMax -2) break;
    }
*/
  }
//###########################################################################################
// отладочные режимы
#else
//-------------------------------------------------------------------------------------------
#if debug_type ==1
//-------------------------------------------------------------------------------------------
  byte Frame[(ARRAY_WIDTH * ARRAY_HEIGHT)+6];

  while(1){
    // grab frame and write to serial
    //reset the pixel grab counter
    ADNS_write(Pixel_Grab,0x00);

    for (int count=0;count < (ARRAY_WIDTH * ARRAY_HEIGHT); count++){
      while(1) {Frame[count] = ADNS_read(Pixel_Grab); if(Frame[count] & Pixel_data_valid) break;}
      Frame[count] &= Mask_pixel_value;  // only n bits are used for data
    }
    Frame[NUM_PIXS] = ADNS_read(squal);
    Frame[NUM_PIXS + 1] = ADNS_read(Maximum_Pixel);
    Frame[NUM_PIXS + 2] = ADNS_read(Minimum_Pixel);
    Frame[NUM_PIXS + 3] = ADNS_read(Pixel_Sum);
    Frame[NUM_PIXS + 4] = ADNS_read(Shutter_Upper);
    Frame[NUM_PIXS + 5] = ADNS_read(Shutter_Lower);

    #if !defined(software_serial)
      Serial.write(Frame, (ARRAY_WIDTH * ARRAY_HEIGHT)+6); // send frame in raw format
    #else
      MyDbgSerial.write(Frame, (ARRAY_WIDTH * ARRAY_HEIGHT)+6);
    #endif

    //delay(2);
  }
//-------------------------------------------------------------------------------------------
#elif debug_type ==2
//-------------------------------------------------------------------------------------------
  byte Frame[(ARRAY_WIDTH * ARRAY_HEIGHT)+6];

  byte data;
  while(1){
    //data = ADNS_read(squal);
    data = ADNS_read(Maximum_Pixel);
    //data = ADNS_read(Pixel_Sum);

    if(data > ConstMax){
      pin_led_LOW;
    }
    else{
      pin_led_HIGH;

      // grab frame and write to serial
      //reset the pixel grab counter
      ADNS_write(Pixel_Grab,0x00);

      for (int count=0;count < (ARRAY_WIDTH * ARRAY_HEIGHT); count++){
        while(1) {Frame[count] = ADNS_read(Pixel_Grab); if(Frame[count] & Pixel_data_valid) break;}
        Frame[count] &= Mask_pixel_value;  // only n bits are used for data
      }
      Frame[NUM_PIXS] = ADNS_read(squal);
      Frame[NUM_PIXS + 1] = ADNS_read(Maximum_Pixel);
      Frame[NUM_PIXS + 2] = ADNS_read(Minimum_Pixel);
      Frame[NUM_PIXS + 3] = ADNS_read(Pixel_Sum);
      Frame[NUM_PIXS + 4] = ADNS_read(Shutter_Upper);
      Frame[NUM_PIXS + 5] = ADNS_read(Shutter_Lower);

      MyDbgSerial.write(Frame, (ARRAY_WIDTH * ARRAY_HEIGHT)+6); // send frame in raw format
    }
  }
//-------------------------------------------------------------------------------------------
#elif debug_type ==3
//-------------------------------------------------------------------------------------------
  //листинг для электронных таблиц: В шапке названия, дальше только данные разделенные "tab".
  byte Frame[6];
  //заголовок
  #if !defined(software_serial)
    Serial.println      ("squal:\tMax_Pix:\tMin_Pix:\tPix_Sum:\tShutter:");
  #else
    MyDbgSerial.println  (F  ("Squal:\tMax:\tMin:\tSum:\tShutter:"));
  #endif
  while(1){
    Frame[0] = ADNS_read(squal);
    Frame[1] = ADNS_read(Maximum_Pixel);
    Frame[2] = ADNS_read(Minimum_Pixel);
    Frame[3] = ADNS_read(Pixel_Sum);
    Frame[4] = ADNS_read(Shutter_Upper);
    Frame[5] = ADNS_read(Shutter_Lower);

    #if !defined(software_serial)
      ByteToString(Frame[0]); Serial.write(Str[2]); Serial.write(Str[1]); Serial.write(Str[0]); Serial.write(0x09);
      ByteToString(Frame[1]); Serial.write(Str[2]); Serial.write(Str[1]); Serial.write(Str[0]); Serial.write(0x09);
      ByteToString(Frame[2]); Serial.write(Str[2]); Serial.write(Str[1]); Serial.write(Str[0]); Serial.write(0x09);
      ByteToString(Frame[3]); Serial.write(Str[2]); Serial.write(Str[1]); Serial.write(Str[0]); Serial.write(0x09);
      Uint16ToString(Frame[4] *256 + Frame[5]);
        Serial.write(Str[4]);
        Serial.write(Str[3]);
        Serial.write(Str[2]);
        Serial.write(Str[1]);
        Serial.write(Str[0]);
        Serial.write(0x0a);
        //Serial.write(0x0d);
      delay(20);
    #else
      ByteToString(Frame[0]); MyDbgSerial.write(Str[2]); MyDbgSerial.write(Str[1]); MyDbgSerial.write(Str[0]); MyDbgSerial.write(0x09);
      ByteToString(Frame[1]); MyDbgSerial.write(Str[2]); MyDbgSerial.write(Str[1]); MyDbgSerial.write(Str[0]); MyDbgSerial.write(0x09);
      ByteToString(Frame[2]); MyDbgSerial.write(Str[2]); MyDbgSerial.write(Str[1]); MyDbgSerial.write(Str[0]); MyDbgSerial.write(0x09);
      ByteToString(Frame[3]); MyDbgSerial.write(Str[2]); MyDbgSerial.write(Str[1]); MyDbgSerial.write(Str[0]); MyDbgSerial.write(0x09);
      Uint16ToString(Frame[4]*256+Frame[5]);
        MyDbgSerial.write(Str[4]);
        MyDbgSerial.write(Str[3]);
        MyDbgSerial.write(Str[2]);
        MyDbgSerial.write(Str[1]);
        MyDbgSerial.write(Str[0]);
        MyDbgSerial.write(0x0a);
        // MyDbgSerial.write(0x0d);
      delay(60);  //задержка больше из-за низкой скорости Digispark
    #endif
  }
//-------------------------------------------------------------------------------------------
#elif debug_type ==4
//-------------------------------------------------------------------------------------------
  //Как 3-й режим, но по разрешению сигнала pin_TRIG (лог точно ограничен сигналом z_probe)
  byte Frame[6];
  while(1){
    while(!(pin_TRIG_IN)){}  //ждать "1"
    //заголовок
    #if !defined(software_serial)
      Serial.println  ("squal:\tMax_Pix:\tMin_Pix:\tPix_Sum:\tShutter:");
    #else
      MyDbgSerial.println (F  ("Squal:\tMax:\tMin:\tSum:\tShutter:"));
    #endif

    for (uint16_t i = 0; i < NUM_SAMPLES_PER_MEASURE; i++) {
      Frame[0] = ADNS_read(squal);
      Frame[1] = ADNS_read(Maximum_Pixel);
      Frame[2] = ADNS_read(Minimum_Pixel);
      Frame[3] = ADNS_read(Pixel_Sum);
      Frame[4] = ADNS_read(Shutter_Upper);
      Frame[5] = ADNS_read(Shutter_Lower);

    #if !defined(software_serial)
        ByteToString(Frame[0]); Serial.write(Str[2]); Serial.write(Str[1]); Serial.write(Str[0]); Serial.write(0x09);
        ByteToString(Frame[1]); Serial.write(Str[2]); Serial.write(Str[1]); Serial.write(Str[0]); Serial.write(0x09);
        ByteToString(Frame[2]); Serial.write(Str[2]); Serial.write(Str[1]); Serial.write(Str[0]); Serial.write(0x09);
        ByteToString(Frame[3]); Serial.write(Str[2]); Serial.write(Str[1]); Serial.write(Str[0]); Serial.write(0x09);
        Uint16ToString(Frame[4] *256 + Frame[5]);
          Serial.write(Str[4]);
          Serial.write(Str[3]);
          Serial.write(Str[2]);
          Serial.write(Str[1]);
          Serial.write(Str[0]);
          Serial.write(0x0a);
          //Serial.write(0x0d);
        delay(20);
    #else
        ByteToString(Frame[0]); MyDbgSerial.write(Str[2]); MyDbgSerial.write(Str[1]); MyDbgSerial.write(Str[0]); MyDbgSerial.write(0x09);
        ByteToString(Frame[1]); MyDbgSerial.write(Str[2]); MyDbgSerial.write(Str[1]); MyDbgSerial.write(Str[0]); MyDbgSerial.write(0x09);
        ByteToString(Frame[2]); MyDbgSerial.write(Str[2]); MyDbgSerial.write(Str[1]); MyDbgSerial.write(Str[0]); MyDbgSerial.write(0x09);
        ByteToString(Frame[3]); MyDbgSerial.write(Str[2]); MyDbgSerial.write(Str[1]); MyDbgSerial.write(Str[0]); MyDbgSerial.write(0x09);
        Uint16ToString(Frame[4] *256 +Frame[5]);
          MyDbgSerial.write(Str[4]);
          MyDbgSerial.write(Str[3]);
          MyDbgSerial.write(Str[2]);
          MyDbgSerial.write(Str[1]);
          MyDbgSerial.write(Str[0]);
          MyDbgSerial.write(0x0a);
          // MyDbgSerial.write(0x0d);
        delay(60);  //задержка больше из-за низкой скорости Digispark
    #endif
    }
    while(pin_TRIG_IN){}  //ждать "0"
  }
//-------------------------------------------------------------------------------------------
#elif debug_type ==5
//-------------------------------------------------------------------------------------------
  while(1){
    unsigned int motion = 0;
    //check for movement
    motion = ADNS_read(Motion);
    if(motion > 127){
      //there has been a motion! Print the x and y movements
      MyDbgSerial.println("X:" + String(ADNS_read(Delta_X)) + " Y:" + String(ADNS_read(Delta_Y)));
    }
  }
//-------------------------------------------------------------------------------------------
#endif
#endif
}

//###########################################################################################
// процедуры
//-------------------------------------------------------------------------------------------
void ADNS_reset(void){
#ifdef flg_ADNS_type_ADNS_5020
  ADNS_write(0x3a,0x5a);
  delay(1000);
  ADNS_write(0x0d, 0x01);  // Set 1000cpi resolution
  delay(1000);
#endif
#ifdef flg_ADNS_type_ADNS_2610
  //ADNS_write(0x00,0x80);
  delay(1000);
  ADNS_write(0x00,0x01); //Always awake
#endif
}

//-------------------------------------------------------------------------------------------
void ADNS_write(byte address, byte data){
  #ifdef use_nCS
    pin_nCS_LOW;
  #endif
  // send in the address and value via SPI:
  address |=0x80;  //признак записи адреса
  for(byte i = 0x80; i; i >>= 1){
    pin_nClock_LOW;
    address & i ? pin_SDIO_HIGH : pin_SDIO_LOW;
    asm volatile ("nop");
    pin_nClock_HIGH;
  }

  //delayMicroseconds(1);

  for(byte i = 0x80; i; i >>= 1){
    pin_nClock_LOW;
    data & i ? pin_SDIO_HIGH : pin_SDIO_LOW;
    asm volatile ("nop");
    pin_nClock_HIGH;
  }
  //t SWW. SPI Time between Write Commands
  delayMicroseconds(delay_tSWW);

  #ifdef use_nCS
    pin_nCS_HIGH;
  #endif
}

//-------------------------------------------------------------------------------------------
byte ADNS_read(byte address){
  #ifdef use_nCS
    pin_nCS_LOW;
  #endif

  address &=~0x80;  //признак записи данных
  for(byte i = 0x80; i; i >>= 1){
    pin_nClock_LOW;
    address & i ? pin_SDIO_HIGH : pin_SDIO_LOW;
    asm volatile ("nop");
    pin_nClock_HIGH;
  }

  // prepare io pin for reading
  pin_SDIO_Mode_INPUT;

  // t SRAD. SPI Read Address-Data Delay
  delayMicroseconds(delay_tSRAD);

  // read the data
  byte data = 0;
  for (byte i = 8; i; i--){
    // tick, tock, read
    pin_nClock_LOW;
    asm volatile ("nop");
    pin_nClock_HIGH;
    data <<= 1;
    if(pin_SDIO_IN) data |= 0x01;
  }

  #ifdef use_nCS
    pin_nCS_HIGH;
  #endif
  pin_SDIO_Mode_OUTPUT;

  // t SRW & t SRR = 1μs.
  delayMicroseconds(2);
  return data;
}

//-------------------------------------------------------------------------------------------
void ByteToString(uint8_t a){
  Str[0] = '0';
  Str[1] = '0';
  Str[2] = '0';
  if(!a) return;
  Str[0] = a % 10 + '0'; a /= 10;
  if(!a) return;
  Str[1] = a % 10 + '0'; a /= 10;
  if(!a) return;
  Str[2] = a % 10 + '0';
}
//-------------------------------------------------------------------------------------------
void Uint16ToString(uint16_t a){
  Str[0] = '0';
  Str[1] = '0';
  Str[2] = '0';
  Str[3] = '0';
  Str[4] = '0';
  if(!a) return;
  Str[0] = a % 10 + '0'; a /= 10;
  if(!a) return;
  Str[1] = a % 10 + '0'; a /= 10;
  if(!a) return;
  Str[2] = a % 10 + '0'; a /= 10;
  if(!a) return;
  Str[3] = a % 10 + '0'; a /= 10;
  if(!a) return;
  Str[4] = a % 10 + '0';
}
//-------------------------------------------------------------------------------------------
uint8_t ByteToAscii_h (uint8_t x){
  uint8_t tmpByte;

  tmpByte = (x >>4) + 0x30;
  if (tmpByte > 0x39) {tmpByte +=0x07;}
  return tmpByte;
}
//-------------------------------------------------------------------------------------------
uint8_t ByteToAscii_l (uint8_t x){
  uint8_t tmpByte;
  tmpByte = (x & 0x0f) + 0x30;
  if (tmpByte > 0x39) {tmpByte +=0x07;}
  return tmpByte;
}
